@@ -45,7 +45,8 @@ file_requests = {'os.listdir':0, 'rpath.make_file_dict':0, 'os.chmod':0,
 				 'os.chown':0, 'os.remove':0, 'os.removedirs':0,
 				 'os.rename':0, 'os.renames':0, 'os.rmdir':0, 'os.unlink':0,
 				 'os.utime':0, 'os.lchown':0, 'os.link':1, 'os.symlink':1,
-				 'os.mkdir':0, 'os.makedirs':0}
+				 'os.mkdir':0, 'os.makedirs':0,
+				 'rpath.delete_dir_no_files':0}
 				 
 def initialize(action, cmdpairs):
 	"""Initialize allowable request list and chroot"""
@@ -172,6 +173,7 @@ def set_allowed_requests(sec_level):
 				  "backup.DestinationStruct.patch_and_increment",
 				  "Main.backup_touch_curmirror_local",
 				  "Main.backup_remove_curmirror_local",
+				  "Main.backup_close_statistics",
 				  "regress.check_pids",
 				  "Globals.ITRB.increment_stat",
 				  "statistics.record_error",
@@ -180,6 +182,7 @@ def set_allowed_requests(sec_level):
 	if sec_level == "all":
 		l.extend(["os.mkdir", "os.chown", "os.lchown", "os.rename",
 				  "os.unlink", "os.remove", "os.chmod", "os.makedirs",
+				  "rpath.delete_dir_no_files",
 				  "backup.DestinationStruct.patch",
 				  "restore.TargetStruct.get_initial_iter",
 				  "restore.TargetStruct.patch",
